@@ -5,6 +5,7 @@ from sympy import preview, sympify
 
 from gtrpy.equations.tensorfieldEP import *
 from gtrpy.tools.image_resizer_fields3D import *
+from gtrpy.tools.latex_output import *
 
 
 # Image Path
@@ -36,13 +37,22 @@ def tensorfield_gui3d(event, metric_tensor, coord_sys):
                                         sg.InputText(default_text='0', size=(15, 1),  font=('Tahoma', 11)),
                                         sg.InputText(default_text='0', size=(15, 1), font=('Tahoma', 11))],
 
+                                    [sg.Frame(layout=[
+                                        [sg.Button('Calculate', button_color='purple'),
+                                        sg.Image(resPATH + r'/images3D/tensorfield_11.png'),
+                                        sg.Button('Calculate', button_color='purple'),
+                                        sg.Image(resPATH + r'/images3D/tensorfield_02.png')]],
+                                    title='Vary Type', font=('Verdana', 12), expand_x=True,
+                                    element_justification='center', title_location='n')],
+
                                     [sg.Frame(layout=[
                                         [sg.Button('Calculate', button_color='purple'),
                                         sg.Image(resPATH + r'/images3D/cov_tensorfield_20.png'),
                                         sg.Text('for', font=('Verdana', 11)),
                                         sg.Image(resPATH + r'/images3D/e.png'),
                                         sg.InputCombo(coord_sys, default_value=coord_sys[0])]],
-                                    title='Covariant Derivative', font=('Verdana', 12), expand_x=True, element_justification='center', title_location='n')],
+                                    title='Covariant Derivative', font=('Verdana', 12), expand_x=True,
+                                    element_justification='center', title_location='n')],
 
                                     [sg.Frame(layout=[
                                         [sg.Image(resPATH + r'/images3D/LX0.png'),
@@ -53,7 +63,8 @@ def tensorfield_gui3d(event, metric_tensor, coord_sys):
                                         sg.InputText(default_text='0', font=('Tahoma', 11))],
                                         [sg.Button('Calculate', button_color='purple'),
                                         sg.Image(resPATH + r'/images3D/LX_tensorfield_20.png')]],
-                                    title='Lie Derivative', font=('Verdana', 12), expand_x=True, element_justification='center', title_location='n')]
+                                    title='Lie Derivative', font=('Verdana', 12), expand_x=True,
+                                    element_justification='center', title_location='n')]
                                 ]
         windows_tensor_field = sg.Window('Tensor Field', tensor_field_20_layout)
         while True:
@@ -63,38 +74,79 @@ def tensorfield_gui3d(event, metric_tensor, coord_sys):
             else:
                 tensor_field = [[sympify(values[i+j]) for i in range(3)] for j in range(1, 13, 4)]   # Obtaining the tensor field
 
-                # Calculation of the covariant derivative
+                # Varying tensor field type from 'uu' to 'ud'
                 if event == 'Calculate':
-                    index_symbol = values[14]
+                    vry_tensor_field_eqn = vry_tensorfield20_ep(metric_tensor, coord_sys, tensor_field, new_type='ud')
+                    preview(vry_tensor_field_eqn, viewer='file', filename=r'logs/vry_tensor_field_20.png', euler=True,
+                            dvioptions=['-T', 'tight', '-z', '0', '--truecolor', '-D 1200', '-bg', 'Transparent'])
+                    resize_vry_type_image3d('Type (2,0) Tensor Field')
+                    layout_vry_tensor_field_result = [
+                                                        [sg.Image(r'logs/vry_tensor_field_20.png')],
+                                                        [sg.Button('Get LaTeX', button_color='orange')]
+                                                    ]
+                    window_vry_tensor_field_result = sg.Window('Tensor Field', layout_vry_tensor_field_result)
+                    while True:
+                        event, values = window_vry_tensor_field_result.read()
+                        if event == sg.WIN_CLOSED:
+                            break
+                        if event == 'Get LaTeX':
+                            latex_output_tensor_field(vry_tensor_field_eqn)
+
+                # Varying tensor field type from 'uu' to 'dd'
+                if event == 'Calculate0':
+                    vry_tensor_field_eqn = vry_tensorfield20_ep(metric_tensor, coord_sys, tensor_field, new_type='dd')
+                    preview(vry_tensor_field_eqn, viewer='file', filename=r'logs/vry_tensor_field_20.png', euler=True,
+                            dvioptions=['-T', 'tight', '-z', '0', '--truecolor', '-D 1200', '-bg', 'Transparent'])
+                    resize_vry_type_image3d('Type (2,0) Tensor Field')
+                    layout_vry_tensor_field_result = [
+                                                        [sg.Image(r'logs/vry_tensor_field_20.png')],
+                                                        [sg.Button('Get LaTeX', button_color='orange')]
+                                                    ]
+                    window_vry_tensor_field_result = sg.Window('Tensor Field', layout_vry_tensor_field_result)
+                    while True:
+                        event, values = window_vry_tensor_field_result.read()
+                        if event == sg.WIN_CLOSED:
+                            break
+                        if event == 'Get LaTeX':
+                            latex_output_tensor_field(vry_tensor_field_eqn)
+
+                # Calculation of the covariant derivative
+                if event == 'Calculate1':
+                    index_symbol = values[16]
                     cd_tensor_field_eqn = cd_tensorfield20_ep(metric_tensor, coord_sys, tensor_field, index_symbol)
                     preview(cd_tensor_field_eqn, viewer='file', filename=r'logs/cd_tensor_field_20.png', euler=True,
                             dvioptions=['-T', 'tight', '-z', '0', '--truecolor', '-D 1200', '-bg', 'Transparent'])
                     resize_cd_image3d('Type (2,0) Tensor Field')
                     layout_cd_tensor_field_result = [
                                                         [sg.Image(r'logs/cd_tensor_field_20.png')],
+                                                        [sg.Button('Get LaTeX', button_color='orange')]
                                                     ]
                     window_cd_tensor_field_result = sg.Window('Tensor Field', layout_cd_tensor_field_result)
                     while True:
                         event, values = window_cd_tensor_field_result.read()
                         if event == sg.WIN_CLOSED:
                             break
+                        if event == 'Get LaTeX':
+                            latex_output_tensor_field(cd_tensor_field_eqn)
 
                 # Calculation of the Lie derivative
-                elif event == 'Calculate0':
-                    X = [sympify(values[i]) for i in range(16, 22, 2)]
+                elif event == 'Calculate2':
+                    X = [sympify(values[i]) for i in range(18, 24, 2)]
                     ld_tensor_field_eqn = ld_tensorfield20_ep(metric_tensor, coord_sys, tensor_field, X)
                     preview(ld_tensor_field_eqn, viewer='file', filename=r'logs/ld_tensor_field_20.png', euler=True,
                             dvioptions=['-T', 'tight', '-z', '0', '--truecolor', '-D 1200', '-bg', 'Transparent'])
                     resize_ld_image3d('Type (2,0) Tensor Field')
                     layout_ld_tensor_field_result = [
                                                         [sg.Image(r'logs/ld_tensor_field_20.png')],
+                                                        [sg.Button('Get LaTeX', button_color='orange')]
                                                     ]
                     window_ld_tensor_field_result = sg.Window('Tensor Field', layout_ld_tensor_field_result)
                     while True:
                         event, values = window_ld_tensor_field_result.read()
                         if event == sg.WIN_CLOSED:
                             break
-
+                        if event == 'Get LaTeX':
+                            latex_output_tensor_field(ld_tensor_field_eqn)
     elif event == 'Type (1,1) Tensor Field':
         tensor_field_11_layout =  [
                                     [sg.Image(resPATH + r'/images3D/tensorfield_11_0.png'),
@@ -110,13 +162,22 @@ def tensorfield_gui3d(event, metric_tensor, coord_sys):
                                         sg.InputText(default_text='0', size=(15, 1),  font=('Tahoma', 11)),
                                         sg.InputText(default_text='0', size=(15, 1), font=('Tahoma', 11))],
 
+                                    [sg.Frame(layout=[
+                                        [sg.Button('Calculate', button_color='purple'),
+                                        sg.Image(resPATH + r'/images3D/tensorfield_20.png'),
+                                        sg.Button('Calculate', button_color='purple'),
+                                        sg.Image(resPATH + r'/images3D/tensorfield_02.png')]],
+                                    title='Vary Type', font=('Verdana', 12), expand_x=True,
+                                    element_justification='center', title_location='n')],
+
                                     [sg.Frame(layout=[
                                         [sg.Button('Calculate', button_color='purple'),
                                         sg.Image(resPATH + r'/images3D/cov_tensorfield_11.png'),
                                         sg.Text('for', font=('Verdana', 11)),
                                         sg.Image(resPATH + r'/images3D/e.png'),
                                         sg.InputCombo(coord_sys, default_value=coord_sys[0])]],
-                                    title='Covariant Derivative', font=('Verdana', 12), expand_x=True, element_justification='center', title_location='n')],
+                                    title='Covariant Derivative', font=('Verdana', 12), expand_x=True,
+                                    element_justification='center', title_location='n')],
 
                                     [sg.Frame(layout=[
                                         [sg.Image(resPATH + r'/images3D/LX0.png'),
@@ -127,7 +188,8 @@ def tensorfield_gui3d(event, metric_tensor, coord_sys):
                                         sg.InputText(default_text='0', font=('Tahoma', 11))],
                                         [sg.Button('Calculate', button_color='purple'),
                                         sg.Image(resPATH + r'/images3D/LX_tensorfield_11.png')]],
-                                    title='Lie Derivative', font=('Verdana', 12), expand_x=True, element_justification='center', title_location='n')]
+                                    title='Lie Derivative', font=('Verdana', 12), expand_x=True,
+                                    element_justification='center', title_location='n')]
                                     ]
         windows_tensor_field = sg.Window('Tensor Field', tensor_field_11_layout)
         while True:
@@ -137,37 +199,79 @@ def tensorfield_gui3d(event, metric_tensor, coord_sys):
             else:
                 tensor_field = [[sympify(values[i+j]) for i in range(3)] for j in range(1, 13, 4)]   # Obtaining the tensor field
 
-                # Calculation of the covariant derivative
+                # Varying tensor field type from 'ud' to 'uu'
                 if event == 'Calculate':
-                    index_symbol = values[14]
+                    vry_tensor_field_eqn = vry_tensorfield11_ep(metric_tensor, coord_sys, tensor_field, new_type='uu')
+                    preview(vry_tensor_field_eqn, viewer='file', filename=r'logs/vry_tensor_field_11.png', euler=True,
+                            dvioptions=['-T', 'tight', '-z', '0', '--truecolor', '-D 1200', '-bg', 'Transparent'])
+                    resize_vry_type_image3d('Type (1,1) Tensor Field')
+                    layout_vry_tensor_field_result = [
+                                                        [sg.Image(r'logs/vry_tensor_field_11.png')],
+                                                        [sg.Button('Get LaTeX', button_color='orange')]
+                                                    ]
+                    window_vry_tensor_field_result = sg.Window('Tensor Field', layout_vry_tensor_field_result)
+                    while True:
+                        event, values = window_vry_tensor_field_result.read()
+                        if event == sg.WIN_CLOSED:
+                            break
+                        if event == 'Get LaTeX':
+                            latex_output_tensor_field(vry_tensor_field_eqn)
+
+                # Varying tensor field type from 'ud' to 'dd'
+                if event == 'Calculate0':
+                    vry_tensor_field_eqn = vry_tensorfield11_ep(metric_tensor, coord_sys, tensor_field, new_type='dd')
+                    preview(vry_tensor_field_eqn, viewer='file', filename=r'logs/vry_tensor_field_11.png', euler=True,
+                            dvioptions=['-T', 'tight', '-z', '0', '--truecolor', '-D 1200', '-bg', 'Transparent'])
+                    resize_vry_type_image3d('Type (1,1) Tensor Field')
+                    layout_vry_tensor_field_result = [
+                                                        [sg.Image(r'logs/vry_tensor_field_11.png')],
+                                                        [sg.Button('Get LaTeX', button_color='orange')]
+                                                    ]
+                    window_vry_tensor_field_result = sg.Window('Tensor Field', layout_vry_tensor_field_result)
+                    while True:
+                        event, values = window_vry_tensor_field_result.read()
+                        if event == sg.WIN_CLOSED:
+                            break
+                        if event == 'Get LaTeX':
+                            latex_output_tensor_field(vry_tensor_field_eqn)
+
+                # Calculation of the covariant derivative
+                if event == 'Calculate1':
+                    index_symbol = values[16]
                     cd_tensor_field_eqn = cd_tensorfield11_ep(metric_tensor, coord_sys, tensor_field, index_symbol)
                     preview(cd_tensor_field_eqn, viewer='file', filename=r'logs/cd_tensor_field_11.png', euler=True,
                             dvioptions=['-T', 'tight', '-z', '0', '--truecolor', '-D 1200', '-bg', 'Transparent'])
                     resize_cd_image3d('Type (1,1) Tensor Field')
                     layout_cd_tensor_field_result = [
                                                         [sg.Image(r'logs/cd_tensor_field_11.png')],
+                                                        [sg.Button('Get LaTeX', button_color='orange')]
                                                     ]
                     window_cd_tensor_field_result = sg.Window('Tensor Field', layout_cd_tensor_field_result)
                     while True:
                         event, values = window_cd_tensor_field_result.read()
                         if event == sg.WIN_CLOSED:
                             break
+                        if event == 'Get LaTeX':
+                            latex_output_tensor_field(cd_tensor_field_eqn)
 
                 # Calculation of the Lie derivative
-                elif event == 'Calculate0':
-                    X = [sympify(values[i]) for i in range(16, 22, 2)]
+                elif event == 'Calculate2':
+                    X = [sympify(values[i]) for i in range(18, 24, 2)]
                     ld_tensor_field_eqn = ld_tensorfield11_ep(metric_tensor, coord_sys, tensor_field, X)
                     preview(ld_tensor_field_eqn, viewer='file', filename=r'logs/ld_tensor_field_11.png', euler=True,
                             dvioptions=['-T', 'tight', '-z', '0', '--truecolor', '-D 1200', '-bg', 'Transparent'])
                     resize_ld_image3d('Type (1,1) Tensor Field')
                     layout_ld_tensor_field_result = [
                                                         [sg.Image(r'logs/ld_tensor_field_11.png')],
+                                                        [sg.Button('Get LaTeX', button_color='orange')]
                                                     ]
                     window_ld_tensor_field_result = sg.Window('Tensor Field', layout_ld_tensor_field_result)
                     while True:
                         event, values = window_ld_tensor_field_result.read()
                         if event == sg.WIN_CLOSED:
                             break
+                        if event == 'Get LaTeX':
+                            latex_output_tensor_field(ld_tensor_field_eqn)
     else:
         tensor_field_02_layout =  [
                                     [sg.Image(resPATH + r'/images3D/tensorfield_02_0.png'),
@@ -183,13 +287,22 @@ def tensorfield_gui3d(event, metric_tensor, coord_sys):
                                         sg.InputText(default_text='0', size=(15, 1),  font=('Tahoma', 11)),
                                         sg.InputText(default_text='0', size=(15, 1), font=('Tahoma', 11))],
 
+                                    [sg.Frame(layout=[
+                                        [sg.Button('Calculate', button_color='purple'),
+                                        sg.Image(resPATH + r'/images3D/tensorfield_11.png'),
+                                        sg.Button('Calculate', button_color='purple'),
+                                        sg.Image(resPATH + r'/images3D/tensorfield_20.png')]],
+                                    title='Vary Type', font=('Verdana', 12), expand_x=True,
+                                    element_justification='center', title_location='n')],
+
                                     [sg.Frame(layout=[
                                         [sg.Button('Calculate', button_color='purple'),
                                         sg.Image(resPATH + r'/images3D/cov_tensorfield_02.png'),
                                         sg.Text('for', font=('Verdana', 11)),
                                         sg.Image(resPATH + r'/images3D/e.png'),
                                         sg.InputCombo(coord_sys, default_value=coord_sys[0])]],
-                                    title='Covariant Derivative', font=('Verdana', 12), expand_x=True, element_justification='center', title_location='n')],
+                                    title='Covariant Derivative', font=('Verdana', 12), expand_x=True,
+                                    element_justification='center', title_location='n')],
 
                                     [sg.Frame(layout=[
                                         [sg.Image(resPATH + r'/images3D/LX0.png'),
@@ -200,7 +313,8 @@ def tensorfield_gui3d(event, metric_tensor, coord_sys):
                                         sg.InputText(default_text='0', font=('Tahoma', 11))],
                                         [sg.Button('Calculate', button_color='purple'),
                                         sg.Image(resPATH + r'/images3D/LX_tensorfield_02.png')]],
-                                    title='Lie Derivative', font=('Verdana', 12), expand_x=True, element_justification='center', title_location='n')]
+                                    title='Lie Derivative', font=('Verdana', 12), expand_x=True,
+                                    element_justification='center', title_location='n')]
                                     ]
         windows_tensor_field = sg.Window('Tensor Field', tensor_field_02_layout)
         while True:
@@ -210,34 +324,76 @@ def tensorfield_gui3d(event, metric_tensor, coord_sys):
             else:
                 tensor_field = [[sympify(values[i+j]) for i in range(3)] for j in range(1, 13, 4)]   # Obtaining the tensor field
 
-                # Calculation of the covariant derivative
+                # Varying tensor field type from 'dd' to 'ud'
                 if event == 'Calculate':
-                    index_symbol = values[14]
+                    vry_tensor_field_eqn = vry_tensorfield02_ep(metric_tensor, coord_sys, tensor_field, new_type='ud')
+                    preview(vry_tensor_field_eqn, viewer='file', filename=r'logs/vry_tensor_field_02.png', euler=True,
+                            dvioptions=['-T', 'tight', '-z', '0', '--truecolor', '-D 1200', '-bg', 'Transparent'])
+                    resize_vry_type_image3d('Type (0,2) Tensor Field')
+                    layout_vry_tensor_field_result = [
+                                                        [sg.Image(r'logs/vry_tensor_field_02.png')],
+                                                        [sg.Button('Get LaTeX', button_color='orange')]
+                                                    ]
+                    window_vry_tensor_field_result = sg.Window('Tensor Field', layout_vry_tensor_field_result)
+                    while True:
+                        event, values = window_vry_tensor_field_result.read()
+                        if event == sg.WIN_CLOSED:
+                            break
+                        if event == 'Get LaTeX':
+                            latex_output_tensor_field(vry_tensor_field_eqn)
+
+                # Varying tensor field type from 'dd' to 'uu'
+                if event == 'Calculate0':
+                    vry_tensor_field_eqn = vry_tensorfield02_ep(metric_tensor, coord_sys, tensor_field, new_type='uu')
+                    preview(vry_tensor_field_eqn, viewer='file', filename=r'logs/vry_tensor_field_02.png', euler=True,
+                            dvioptions=['-T', 'tight', '-z', '0', '--truecolor', '-D 1200', '-bg', 'Transparent'])
+                    resize_vry_type_image3d('Type (0,2) Tensor Field')
+                    layout_vry_tensor_field_result = [
+                                                        [sg.Image(r'logs/vry_tensor_field_02.png')],
+                                                        [sg.Button('Get LaTeX', button_color='orange')]
+                                                    ]
+                    window_vry_tensor_field_result = sg.Window('Tensor Field', layout_vry_tensor_field_result)
+                    while True:
+                        event, values = window_vry_tensor_field_result.read()
+                        if event == sg.WIN_CLOSED:
+                            break
+                        if event == 'Get LaTeX':
+                            latex_output_tensor_field(vry_tensor_field_eqn)
+
+                # Calculation of the covariant derivative
+                if event == 'Calculate1':
+                    index_symbol = values[16]
                     cd_tensor_field_eqn = cd_tensorfield02_ep(metric_tensor, coord_sys, tensor_field, index_symbol)
                     preview(cd_tensor_field_eqn, viewer='file', filename=r'logs/cd_tensor_field_02.png', euler=True,
                             dvioptions=['-T', 'tight', '-z', '0', '--truecolor', '-D 1200', '-bg', 'Transparent'])
                     resize_cd_image3d('Type (0,2) Tensor Field')
                     layout_cd_tensor_field_result = [
                                                         [sg.Image(r'logs/cd_tensor_field_02.png')],
+                                                        [sg.Button('Get LaTeX', button_color='orange')]
                                                     ]
                     window_cd_tensor_field_result = sg.Window('Tensor Field', layout_cd_tensor_field_result)
                     while True:
                         event, values = window_cd_tensor_field_result.read()
                         if event == sg.WIN_CLOSED:
                             break
+                        if event == 'Get LaTeX':
+                            latex_output_tensor_field(cd_tensor_field_eqn)
 
                 # Calculation of the Lie derivative
-                elif event == 'Calculate0':
-                    X = [sympify(values[i]) for i in range(16, 22, 2)]
+                elif event == 'Calculate2':
+                    X = [sympify(values[i]) for i in range(18, 24, 2)]
                     ld_tensor_field_eqn = ld_tensorfield02_ep(metric_tensor, coord_sys, tensor_field, X)
                     preview(ld_tensor_field_eqn, viewer='file', filename=r'logs/ld_tensor_field_02.png', euler=True,
                             dvioptions=['-T', 'tight', '-z', '0', '--truecolor', '-D 1200', '-bg', 'Transparent'])
                     resize_ld_image3d('Type (0,2) Tensor Field')
                     layout_ld_tensor_field_result = [
                                                         [sg.Image(r'logs/ld_tensor_field_02.png')],
+                                                        [sg.Button('Get LaTeX', button_color='orange')]
                                                     ]
                     window_ld_tensor_field_result = sg.Window('Tensor Field', layout_ld_tensor_field_result)
                     while True:
                         event, values = window_ld_tensor_field_result.read()
                         if event == sg.WIN_CLOSED:
                             break
+                        if event == 'Get LaTeX':
+                            latex_output_tensor_field(ld_tensor_field_eqn)
