@@ -5,6 +5,7 @@ from sympy import preview, sympify
 
 from gtrpy.equations.vectorfieldEP import *
 from gtrpy.tools.image_resizer_fields3D import *
+from gtrpy.tools.latex_output import *
 
 
 # Image Path
@@ -33,7 +34,8 @@ def vectorfield_gui3d(event, metric_tensor, coord_sys):
                                     [sg.Frame(layout=[
                                         [sg.Button('Calculate', button_color='purple'),
                                         sg.Image(resPATH + r'/images3D/vectorfield_01.png')]],
-                                    title='Vary Type', font=('Verdana', 12), expand_x=True, element_justification='center', title_location='n')],
+                                    title='Vary Type', font=('Verdana', 12), expand_x=True,
+                                    element_justification='center', title_location='n')],
 
                                     [sg.Frame(layout=[
                                         [sg.Button('Calculate', button_color='purple'),
@@ -41,7 +43,8 @@ def vectorfield_gui3d(event, metric_tensor, coord_sys):
                                         sg.Text('for', font=('Verdana', 11)),
                                         sg.Image(resPATH + r'/images3D/e.png'),
                                         sg.InputCombo(coord_sys, default_value=coord_sys[0])]],
-                                    title='Covariant Derivative', font=('Verdana', 12), expand_x=True, element_justification='center', title_location='n')],
+                                    title='Covariant Derivative', font=('Verdana', 12), expand_x=True,
+                                    element_justification='center', title_location='n')],
 
                                     [sg.Frame(layout=[
                                         [sg.Image(resPATH + r'/images3D/LX0.png'),
@@ -52,7 +55,8 @@ def vectorfield_gui3d(event, metric_tensor, coord_sys):
                                         sg.InputText(default_text='0', font=('Tahoma', 11))],
                                         [sg.Button('Calculate', button_color='purple'),
                                         sg.Image(resPATH + r'/images3D/LX_vectorfield_10.png')]],
-                                    title='Lie Derivative', font=('Verdana', 12), expand_x=True, element_justification='center', title_location='n')]
+                                    title='Lie Derivative', font=('Verdana', 12), expand_x=True,
+                                    element_justification='center', title_location='n')]
                                     ]
         windows_vector_field = sg.Window('Vector Field', vector_field_10_layout)
         while True:
@@ -67,15 +71,18 @@ def vectorfield_gui3d(event, metric_tensor, coord_sys):
                     vry_vector_field_eqn = vry_vectorfield10_ep(metric_tensor, coord_sys, vector_field)
                     preview(vry_vector_field_eqn, viewer='file', filename=r'logs/vry_vector_field_10.png', euler=True,
                             dvioptions=['-T', 'tight', '-z', '0', '--truecolor', '-D 1200', '-bg', 'Transparent'])
-                    resize_vry_image3d('Type (1,0) Vector Field')
+                    resize_vry_type_image3d('Type (1,0) Vector Field')
                     layout_vry_vector_field_result = [
                                                         [sg.Image(r'logs/vry_vector_field_10.png')],
+                                                        [sg.Button('Get LaTeX', button_color='orange')]
                                                     ]
                     window_vry_vector_field_result = sg.Window('Vector Field', layout_vry_vector_field_result)
                     while True:
                         event, values = window_vry_vector_field_result.read()
                         if event == sg.WIN_CLOSED:
                             break
+                        if event == 'Get LaTeX':
+                            latex_output_vector_field(vry_vector_field_eqn)
 
                 # Calculation of the covariant derivative
                 if event == 'Calculate0':
@@ -86,12 +93,15 @@ def vectorfield_gui3d(event, metric_tensor, coord_sys):
                     resize_cd_image3d('Type (1,0) Vector Field')
                     layout_cd_vector_field_result = [
                                                         [sg.Image(r'logs/cd_vector_field_10.png')],
+                                                        [sg.Button('Get LaTeX', button_color='orange')]
                                                     ]
                     window_cd_vector_field_result = sg.Window('Vector Field', layout_cd_vector_field_result)
                     while True:
                         event, values = window_cd_vector_field_result.read()
                         if event == sg.WIN_CLOSED:
                             break
+                        if event == 'Get LaTeX':
+                            latex_output_vector_field(cd_vector_field_eqn)
 
                 # Calculation of the Lie derivative
                 elif event == 'Calculate1':
@@ -102,12 +112,15 @@ def vectorfield_gui3d(event, metric_tensor, coord_sys):
                     resize_ld_image3d('Type (1,0) Vector Field')
                     layout_ld_vector_field_result = [
                                                         [sg.Image(r'logs/ld_vector_field_10.png')],
+                                                        [sg.Button('Get LaTeX', button_color='orange')]
                                                     ]
                     window_ld_vector_field_result = sg.Window('Vector Field', layout_ld_vector_field_result)
                     while True:
                         event, values = window_ld_vector_field_result.read()
                         if event == sg.WIN_CLOSED:
                             break
+                        if event == 'Get LaTeX':
+                            latex_output_vector_field(ld_vector_field_eqn)
     else:
         vector_field_01_layout = [
                                     [sg.Image(resPATH + r'/images3D/vectorfield_01_0.png'),
@@ -120,7 +133,8 @@ def vectorfield_gui3d(event, metric_tensor, coord_sys):
                                     [sg.Frame(layout=[
                                             [sg.Button('Calculate', button_color='purple'),
                                             sg.Image(resPATH + r'/images3D/vectorfield_10.png')]],
-                                    title='Vary Type', font=('Verdana', 12), expand_x=True, element_justification='center', title_location='n')],
+                                    title='Vary Type', font=('Verdana', 12), expand_x=True,
+                                    element_justification='center', title_location='n')],
 
                                     [sg.Frame(layout=[
                                         [sg.Button('Calculate', button_color='purple'),
@@ -128,7 +142,8 @@ def vectorfield_gui3d(event, metric_tensor, coord_sys):
                                         sg.Text('for', font=('Verdana', 11)),
                                         sg.Image(resPATH + r'/images3D/e.png'),
                                         sg.InputCombo(coord_sys, default_value=coord_sys[0])]],
-                                    title='Covariant Derivative', font=('Verdana', 12), expand_x=True, element_justification='center', title_location='n')],
+                                    title='Covariant Derivative', font=('Verdana', 12), expand_x=True,
+                                    element_justification='center', title_location='n')],
 
                                     [sg.Frame(layout=[
                                         [sg.Image(resPATH + r'/images3D/LX0.png'),
@@ -139,7 +154,8 @@ def vectorfield_gui3d(event, metric_tensor, coord_sys):
                                         sg.InputText(default_text='0', font=('Tahoma', 11))],
                                         [sg.Button('Calculate', button_color='purple'),
                                         sg.Image(resPATH + r'/images3D/LX_vectorfield_01.png')]],
-                                    title='Lie Derivative', font=('Verdana', 12), expand_x=True, element_justification='center', title_location='n')]
+                                    title='Lie Derivative', font=('Verdana', 12), expand_x=True,
+                                    element_justification='center', title_location='n')]
                                 ]
         windows_vector_field = sg.Window('Vector Field', vector_field_01_layout)
         while True:
@@ -154,15 +170,18 @@ def vectorfield_gui3d(event, metric_tensor, coord_sys):
                     vry_vector_field_eqn = vry_vectorfield01_ep(metric_tensor, coord_sys, vector_field)
                     preview(vry_vector_field_eqn, viewer='file', filename=r'logs/vry_vector_field_01.png', euler=True,
                             dvioptions=['-T', 'tight', '-z', '0', '--truecolor', '-D 1200', '-bg', 'Transparent'])
-                    resize_vry_image3d('Type (0,1) Vector Field')
+                    resize_vry_type_image3d('Type (0,1) Vector Field')
                     layout_vry_vector_field_result = [
                                                         [sg.Image(r'logs/vry_vector_field_01.png')],
+                                                        [sg.Button('Get LaTeX', button_color='orange')]
                                                     ]
                     window_vry_vector_field_result = sg.Window('Vector Field', layout_vry_vector_field_result)
                     while True:
                         event, values = window_vry_vector_field_result.read()
                         if event == sg.WIN_CLOSED:
                             break
+                        if event == 'Get LaTeX':
+                            latex_output_vector_field(vry_vector_field_eqn)
 
                 # Calculation of the covariant derivative
                 if event == 'Calculate0':
@@ -173,12 +192,15 @@ def vectorfield_gui3d(event, metric_tensor, coord_sys):
                     resize_cd_image3d('Type (0,1) Vector Field')
                     layout_cd_vector_field_result = [
                                                         [sg.Image(r'logs/cd_vector_field_01.png')],
+                                                        [sg.Button('Get LaTeX', button_color='orange')]
                                                     ]
                     window_cd_vector_field_result = sg.Window('Vector Field', layout_cd_vector_field_result)
                     while True:
                         event, values = window_cd_vector_field_result.read()
                         if event == sg.WIN_CLOSED:
                             break
+                        if event == 'Get LaTeX':
+                            latex_output_vector_field(cd_vector_field_eqn)
 
                 # Calculation of the Lie derivative
                 elif event == 'Calculate1':
@@ -189,9 +211,12 @@ def vectorfield_gui3d(event, metric_tensor, coord_sys):
                     resize_ld_image3d('Type (0,1) Vector Field')
                     layout_ld_vector_field_result = [
                                                         [sg.Image(r'logs/ld_vector_field_01.png')],
+                                                        [sg.Button('Get LaTeX', button_color='orange')]
                                                     ]
                     window_ld_vector_field_result = sg.Window('Vector Field', layout_ld_vector_field_result)
                     while True:
                         event, values = window_ld_vector_field_result.read()
                         if event == sg.WIN_CLOSED:
                             break
+                        if event == 'Get LaTeX':
+                            latex_output_vector_field(ld_vector_field_eqn)
